package com.loan.system.service;

import com.loan.system.domain.entity.Step;
import com.loan.system.domain.vo.StepVO;

import java.util.List;

public interface StepService {
    List<StepVO> addStepByCaseId(Long caseId);

    void updateStatusByCaseId(String msg,String stepName, Long caseId);
    
    void cancelPreApprove(Long caseId);

    StepVO updateUserByCaseId(String stepName, Long approverId, Long caseId);

    List<StepVO> getStepByCaseId(Long caseId);

    StepVO findByStepNameAndCaseId(String stepName,Long caseId);

    void updateUserId1ByCaseIdAndStepName(String stepName, Long approverId, Long caseId);

    void updateUserId2ByCaseIdAndStepName(String stepName, Long approverId, Long caseId);

    void logic_delete(Long id);
}